import { realpathSync, readJsonSync, existsSync } from 'fs-extra';
import { resolve } from 'path';
import { moduleFileExtensions } from './constants';

/**
 * 项目根目录
 */
export const rootPath = realpathSync(process.cwd());

/**
 * 将相对于项目根目录的路径解析为绝对路径
 *
 * @param relativePath 相对路径
 */
export const resolveRoot = (relativePath: string) =>
  resolve(rootPath, relativePath);

/**
 * 项目配置
 */
export const appPackageInfo = readJsonSync(resolveRoot('package.json'));

/**
 * 项目版本号
 */
export const VERSION = appPackageInfo.version;

/**
 * 解析模块的真实文件路径
 *
 * @param resolveFn 相对路径解析函数
 * @param filePath 模块相对路径
 */
export const resolveModule = (
  resolveFn: (relativePath: string) => string,
  filePath: string,
) => {
  const extension = moduleFileExtensions.find((_) =>
    existsSync(resolveFn(`${filePath}.${_}`)),
  );

  if (extension) {
    return resolveFn(`${filePath}.${extension}`);
  }

  return resolveFn(`${filePath}.ts`);
};

/**
 * 源码根路径
 */
export const src = resolveRoot(rootPath);

/**
 * 入口文件路径
 */
export const entry = resolveModule(resolveRoot, 'src/index');

/**
 * 测试环境初始化文件路径
 */
export const testsSetup = resolveModule(resolveRoot, 'src/setupTests');

/**
 * 库对应的全局名称
 */
export const globals = {
  react: 'React',
  'react-native': 'ReactNative',
  ...appPackageInfo.globals,
};
